/*
 * Copyright (c) ROSA Laboratory 2011
 * License: GPLv3
 *
 * Authors:
 * Alexey Yermakov <alexey.yermakov@rosalab.ru>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3,
 *   or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "button.h"

#include <QApplication>
#include <QObject>
#include <QWidget>

#define ICON_SIZE 64

AppButton::AppButton(QIcon icon, QString name, QString description, QString desktopFile, QGraphicsItem *parent, Qt::WindowFlags wFlags)
  : QGraphicsWidget(parent, wFlags),
    m_process(0)
{
  m_layout = new QGraphicsLinearLayout(Qt::Vertical);
  m_layout->setContentsMargins(16, 0, 16, 0);
  setLayout(m_layout);

  setContentsMargins(0, 8, 0, 8);
  
  setAcceptHoverEvents(true);
  setAcceptedMouseButtons(Qt::LeftButton);

  m_icon = new Plasma::IconWidget();
  m_label = new Plasma::Label();

  m_layout->addItem(m_icon);
  m_layout->addItem(m_label);

  m_layout->setAlignment(m_icon, Qt::AlignHCenter);

  m_desktopFile = desktopFile;

  m_icon->setIcon(icon);
  m_label->setText(name);

  m_icon->setAcceptHoverEvents(false);
  m_icon->setAcceptedMouseButtons(Qt::NoButton);

  m_label->setAlignment(Qt::AlignHCenter);
  m_label->setWordWrap(true);
  m_label->setMaximumWidth(ICON_SIZE + 32);
  m_label->setMinimumWidth(ICON_SIZE + 32);

  int current_height = m_label->geometry().height();

  m_label->setMinimumHeight(current_height * 3);
  m_label->setMaximumHeight(current_height * 3);
  
  m_icon->setMaximumWidth(ICON_SIZE);
  m_icon->setMinimumWidth(ICON_SIZE);
  m_icon->setMaximumHeight(ICON_SIZE);
  m_icon->setMinimumHeight(ICON_SIZE);

}

AppButton::~AppButton()
{
}

void AppButton::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
}

void AppButton::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{
  QString program = "/usr/bin/kioclient";
  QStringList arguments;

  arguments << "exec" << m_desktopFile;

  if(m_process == 0)
    {
      m_process = new QProcess(this);

      m_process->start(program, arguments);
      m_process->waitForStarted();
    }
  else if(m_process->state() == QProcess::NotRunning)
    {
      delete m_process;
      
      m_process = new QProcess(this);

      m_process->start(program, arguments);
      m_process->waitForStarted();
    }

  kDebug() << m_desktopFile;
  emit appLaunched(m_desktopFile);

  QMetaObject::invokeMethod(static_cast<QObject*>(QApplication::activeWindow()), "hideWindow");
}

void AppButton::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  //m_icon->hoverEnterEvent(event);
  emit hoverEnter(static_cast<QGraphicsWidget*>(this));
}

void AppButton::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
  //m_icon->hoverLeaveEvent(event);
  emit hoverLeave(static_cast<QGraphicsWidget*>(this));
}
