/*
 * Copyright (c) ROSA Laboratory 2011
 * License: GPLv3
 *
 * Authors:
 * Alexey Yermakov <alexey.yermakov@rosalab.ru>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3,
 *   or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "tabbar.h"

#include <KDebug>
#include <Plasma/Label>

#include <QObject>
#include <QMetaObject>

TabBar::TabBar(QGraphicsItem *parent, Qt::WindowFlags wFlags)
  : QGraphicsWidget(parent, wFlags)
{
  m_layout = new QGraphicsLinearLayout(Qt::Vertical);

  setLayout(m_layout);

  m_tabBar = new Plasma::TabBar();
  m_tabBar->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);

  //m_layout->addStretch();
  m_layout->addItem(m_tabBar);

  m_hiddenWidget = new QGraphicsWidget();
  m_hiddenWidgetLayout = new QGraphicsLinearLayout(Qt::Vertical);
  m_hiddenWidget->setLayout(m_hiddenWidgetLayout);

  m_currentTabWidget = NULL;
  m_currentTabIndex = 0;

  connect(m_tabBar, SIGNAL(currentChanged(int)), this, SLOT(tabChanged(int)));
}

TabBar::~TabBar()
{
}

void TabBar::addTab(QString name, QGraphicsWidget *widget)
{
  m_tabNames.push_back(name);
  m_tabWidgets.push_back(widget);

  m_tabBar->addTab(name);
  m_hiddenWidgetLayout->addItem(widget);

  /////////////
  if(m_currentTabWidget == NULL)
    {
      m_currentTabWidget = widget;
      m_currentTabIndex = 0;
      m_layout->insertItem(0, widget);
    }
}

void TabBar::tabChanged(int index)
{
  //if(m_currentTabWidget != NULL)
  m_hiddenWidgetLayout->addItem(m_currentTabWidget);
  m_currentTabWidget = m_tabWidgets[index];
  m_currentTabIndex = index;
  m_layout->insertItem(0, m_currentTabWidget);

  kDebug() << "Changing tab to " << m_tabNames[index];
  QMetaObject::invokeMethod(m_currentTabWidget, "tabActivated");
  // if(m_currentTabWidget->metaObject()->indexOfMethod("tabActivated") != -1)
  //   {
  //     kDebug() << "Going to call a tabActivated on " << index << " widget";
  //     QMetaObject::invokeMethod(m_currentTabWidget, "tabActivated");
  //   }
  // else
  //   {
  //     kDebug() << "Oops: " << m_currentTabWidget->metaObject()->indexOfMethod("tabActivated");
  //   }
  
}
