/*
 * Copyright (c) ROSA Laboratory 2011
 * License: GPLv3
 *
 * Authors:
 * Alexey Yermakov <alexey.yermakov@rosalab.ru>
 *
 * Based on plasma-windowed by:
 * 2006-2008 Aaron Seigo <aseigo@kde.org>
 * 2009 Marco Martin <notmart@gmail.com>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3,
 *   or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "singleview.h"
#include <iostream>

#include <QAction>
#include <QDir>
#include <QFileInfo>
#include <QDesktopWidget>
#include <QApplication>
#include <QObject>
#include <QApplication>
#include <QBitmap>
#include <QWidget>
#include <QTimer>

#include <KStandardAction>
#include <KAction>
#include <KIconLoader>
#include <KWindowSystem>

#include <Plasma/Applet>
#include <Plasma/Label>
#include <Plasma/Containment>
#include <Plasma/Corona>
#include <Plasma/PopupApplet>
#include <Plasma/WindowEffects>

SingleView::SingleView(Plasma::Corona *corona, Plasma::Containment *containment, QWidget *parent)
    : QGraphicsView(parent),
      m_applet(0),
      m_containment(containment),
      m_corona(corona)
{
  QPalette pal = palette();
  pal.setColor(backgroundRole(), Qt::transparent);
  setPalette(pal);

    setScene(m_corona);

    QString pluginName("rosa-launcher");

    QFileInfo info(pluginName);
    if (!info.isAbsolute()) {
        info = QFileInfo(QDir::currentPath() + '/' + pluginName);
    }

    if (info.exists()) {
        m_applet = Plasma::Applet::loadPlasmoid(info.absoluteFilePath());
    }

    if (!m_applet) {
        m_applet = Plasma::Applet::load(pluginName);
    }

    if (!m_applet) {
        kDebug() << "failed to load" << pluginName;
        return;
    }

    /*
    setBackgroundRole( QPalette::Window );
    setAutoFillBackground( true );
    QColor color = Qt::white;
    color.setAlpha( 0 );
    QPalette p = palette();
    p.setColor( QPalette::Window, QApplication::activeWindow() );
    setPalette( p );
    */


    setAttribute(Qt::WA_TranslucentBackground);
    //setStyleSheet("background: transparent");
    //setAutoFillBackground(false);
    m_containment->setAttribute(Qt::WA_TranslucentBackground);
    m_containment->setAutoFillBackground(false);
        
    m_containment->addApplet(m_applet, QPointF(-1, -1), false);
    QRectF screenRect = QDesktopWidget().screenGeometry();

    resize(screenRect.width() - 20, screenRect.height() - 20);
    move(screenRect.x() + 10, screenRect.y() + 10);

    // m_containment->resize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);

    m_applet->setPos(0, 0);
    m_applet->setFlag(QGraphicsItem::ItemIsMovable, false);
    m_applet->setFocus(Qt::ActiveWindowFocusReason);

    QRectF newSceneRect = m_applet->sceneBoundingRect();

    newSceneRect.setX(newSceneRect.x() + 4);
    newSceneRect.setY(newSceneRect.y() + 4);
    newSceneRect.setWidth(newSceneRect.width() - 8); 
    newSceneRect.setHeight(newSceneRect.height() - 8);
    
    setSceneRect(newSceneRect);
    setWindowTitle(m_applet->name());
    setWindowIcon(SmallIcon(m_applet->icon()));
    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setFrameStyle(QFrame::NoFrame);

    QAction *action = m_applet->action("remove");
    delete action;

    m_applet->addAction(QString("remove"), KStandardAction::quit(this, SLOT(hide()), m_applet));

    // enforce the applet being our size
    connect(m_applet, SIGNAL(geometryChanged()), this, SLOT(updateGeometry()));
    connect(this, SIGNAL(geometryChanged()), m_applet, SLOT(resizeSelf()));
    connect(m_applet, SIGNAL(hideEvent()), this, SLOT(close()));

    updateGeometry();
    QRectF curSceneRect(pos().x(), pos().y(), size().width(), size().height());
    /*
    std::cout << "TEST: " << curSceneRect.x() << ":" << curSceneRect.y() << "; " << curSceneRect.width() << "x" << curSceneRect.height() << ";" << std::endl;
    setSceneRect(0, 0, curSceneRect.width() - curSceneRect.x(), curSceneRect.height() - curSceneRect.y());
    */
   
}

SingleView::~SingleView()
{
    m_containment->destroy(false);
}


void SingleView::setContainment(Plasma::Containment *c)
{
    if (m_containment) {
        disconnect(m_containment, 0, this, 0);
    }

    m_containment = c;
    updateGeometry();
}


void SingleView::resizeEvent(QResizeEvent *event)
{
    Q_UNUSED(event)
    updateGeometry();
    emit geometryChanged();
}

void SingleView::hideEvent(QHideEvent *event)
{
    Q_UNUSED(event)

      /*
    if (m_applet) {
        KConfigGroup dummy;
        m_containment->save(dummy);
        emit storeApplet(m_applet);
        m_applet = 0;
    }

    deleteLater();
      */
}

bool SingleView::toggleWindow()
{
  if(isVisible())
    hideWindow();
  else
    showWindow();

  return isVisible();
}

void SingleView::hideWindow()
{
  hide();
}

void SingleView::showWindow()
{
  KWindowSystem::setOnDesktop(winId(), KWindowSystem::currentDesktop());

  //emit m_containment->proxy->show();
  show();
  activateWindow();
  raise();

  QRectF curSceneRect = sceneRect();
  setSceneRect(0, 0, curSceneRect.width() - curSceneRect.x(), curSceneRect.height() - curSceneRect.y());
}

Plasma::Applet *SingleView::applet()
{
    return m_applet;
}

Plasma::Location SingleView::location() const
{
    return m_containment->location();
}

Plasma::FormFactor SingleView::formFactor() const
{
    return m_containment->formFactor();
}

void SingleView::updateGeometry()
{
    if (!m_containment) {
        return;
    }

    //kDebug() << "New applet geometry is" << m_applet->geometry();

    if (m_applet && m_applet->size().toSize() != size()) {
        if (m_applet) {
            m_applet->resize(size());
        }

        setSceneRect(m_applet->sceneBoundingRect());

    }

    if ((windowFlags() & Qt::FramelessWindowHint) &&
            applet()->backgroundHints() != Plasma::Applet::NoBackground) {

      if(KWindowSystem::compositingActive())
        {
          // TODO: Use the background's mask for blur
          QRegion mask;
          mask += QRect(QPoint(), size());

          Plasma::WindowEffects::enableBlurBehind(winId(), true, mask);
        }
      else
        {
          QTimer::singleShot(0, this, SLOT(updateSnapshot()));
        };
    }
    
}

void SingleView::drawBackground(QPainter* p, const QRectF& rect) {
  //QPainter::CompositionMode c = p->compositionMode();
  //p->setCompositionMode(QPainter::CompositionMode_Source);
  //p->fillRect(rect, Qt::transparent);
  //p->setCompositionMode(c);

  //p->setClipping(true);
  //p->setClipRegion(m_applet->boundingRegion(m_applet->sceneTransform()));
  //p->setClipRect(QRect(0, 0, 100, 100));
  //QPainterPath ppath = m_applet->opaqueArea();
  //p->setClipPath(ppath);

  //p->fillRect(rect, QColor(255, 0, 255, 255));

};

void SingleView::updateSnapshot()
{
  /*
  m_appletSnapshot = QPixmap::grabWidget(this);
  QBitmap mask = m_appletSnapshot.createMaskFromColor(QColor(255, 0, 255, 255));
  m_appletSnapshot.setMask(mask);
  m_appletSnapshot.save("blabla-test.png");
  
  QApplication::activeWindow()->setMask(m_appletSnapshot.mask());
  */
}

bool SingleView::event(QEvent *event)
{
  /*
  kDebug() << "EVENT: " << event->type();
  if(event->type() == QEvent::ApplicationDeactivate)
    {
      QMetaObject::invokeMethod(static_cast<QObject*>(QApplication::activeWindow()), "hideWindow");
    }
  */
  if (event->type() == QEvent::Paint) {
    QPainter p(this);
    p.setCompositionMode(QPainter::CompositionMode_Source);
    p.fillRect(rect(), Qt::transparent);
  }

  return QGraphicsView::event(event);
}

#include "singleview.moc"

